@@ -2,7 +2,7 @@
   utf16_be.c -  Oniguruma (regular expression library)
 **********************************************************************/
 /*-
- * Copyright (c) 2002-2019  K.Kosako
+ * Copyright (c) 2002-2020  K.Kosako
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -128,7 +128,7 @@ static int
 utf16be_is_mbc_newline(const UChar* p, const UChar* end)
 {
   if (p + 1 < end) {
-    if (*(p+1) == 0x0a && *p == 0x00)
+    if (*(p+1) == NEWLINE_CODE && *p == 0x00)
       return 1;
 #ifdef USE_UNICODE_ALL_LINE_TERMINATORS
     if ((
@@ -193,7 +193,7 @@ utf16be_code_to_mbc(OnigCodePoint code, UChar *buf)
   }
   else {
     *p++ = (UChar )((code & 0xff00) >> 8);
-    *p++ = (UChar )(code & 0xff);
+    *p   = (UChar )(code & 0xff);
     return 2;
   }
 }
